#include "main.h"

#include "olympus.c"

#include "cameras.c"
#include "callbacks.c"
#include "plugins.c"
#include "menu.c"
#include "toolbar.c"
#include "commandline.c"

/*
	If you're lookin' at this, then you're curious as to the
	under workings of gPhoto. Start with main.h to see what
	the globals are (eek, globals) then run from there. And yes,
	globals will be completely eliminated in the actual release.
	They're just a little more "convenient" for now. :-)

	I'm working on a common library interface so that adding
	support for more cameras in the future will be a snap.
	For now, you can survive with the PhotoPC library. :-)
			-SF
*/

int main (int argc, char *argv[]) {

	int no_rc=0;

	GtkWidget *mainWin;
	GtkWidget *table;
	GtkWidget *menu_bar;
	GtkWidget *index_page;
	GtkWidget *label, *box, *sbox;
	GtkStyle *style;

	GtkWidget *gpixmap;
	GdkPixmap *pixmap;
	GdkBitmap *bitmap;

	FILE *conf;
	char fname[1024];

	gtk_init(&argc, &argv);
	gdk_imlib_init();
	gtk_widget_push_visual(gdk_imlib_get_visual());
	gtk_widget_push_colormap(gdk_imlib_get_colormap());

	Thumbnails.next = NULL;
	Images.next=NULL;

	/* Set the priority (taken from PhotoPC photopc.c) */
        if (geteuid() == 0) {
                struct sched_param sp;
                int rc,minp,maxp;

                minp=sched_get_priority_min(SCHED_FIFO);
                maxp=sched_get_priority_max(SCHED_FIFO);
                sp.sched_priority=minp+(maxp-minp)/2;
                if ((rc=sched_setscheduler(0,SCHED_FIFO,&sp)) == -1)
                        fprintf(stderr,"failed to set priority\n");
                /* (void)seteuid(getuid()); */
	}

	/* Make sure there's a .gphoto directory in their home ---- */
	gphotoDir = getenv("HOME");
	sprintf(gphotoDir, "%s/.gphoto", gphotoDir);
	(void)mkdir(gphotoDir, 0744);

	/* Load config options------------------------------------ */
	sprintf(fname, "%s/gphotorc", gphotoDir);
	conf = fopen(fname, "r");
	if (!conf)
		no_rc = 1;
	   else {
		fgets(fname, 100, conf);
		strncpy(serial_port, fname, strlen(fname)-1);
		fgets(fname, 100, conf);
		strncpy(camera_model, fname, strlen(fname)-1);
		fclose(conf);
	}
	set_camera(camera_model);

	/* Initialize the camera iobuffer ------------------------ */
	iob = eph_new(NULL, NULL, NULL, NULL, 0);

	/* Command line mode anyone? ----------------------------- */
	if (argc > 1) {
		if (strcmp(argv[1], "-c") == 0)
			command_line(argc, argv);
	}

	/* set up the main window -------------------------------- */
	mainWin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_container_border_width (GTK_CONTAINER(mainWin), 0);
	gtk_window_set_title (GTK_WINDOW(mainWin), TITLE_VER);
	gtk_signal_connect (GTK_OBJECT(mainWin), "delete_event",
			    GTK_SIGNAL_FUNC(delete_event), NULL);
	gtk_widget_set_usize(mainWin, 640, 480);
	gtk_widget_realize(mainWin);

	/* set up the menu --------------------------------------- */
	menu_bar = gtk_menu_bar_new();
	create_menu(menu_bar);
	gtk_widget_show(menu_bar);

	/* button bar -------------------------------------------- */
	box = gtk_hbox_new(FALSE, 0);
	create_toolbar(box, mainWin);
	gtk_widget_show(box);
	gtk_container_border_width(GTK_CONTAINER(box), 5);

	/* Index Page notebook ----------------------------------- */
	index_page = gtk_table_new(1,1,FALSE);
	gtk_widget_show(index_page);
	index_window = gtk_scrolled_window_new(NULL,NULL);
	gtk_widget_show(index_window);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(index_window),
					GTK_POLICY_AUTOMATIC,
					GTK_POLICY_AUTOMATIC);
	gtk_table_attach_defaults(GTK_TABLE(index_page),index_window,0,1,0,1);

	notebook = gtk_notebook_new();
	gtk_widget_show(notebook);
	gtk_notebook_set_scrollable(GTK_NOTEBOOK(notebook), TRUE);

	label = gtk_label_new("Image Index");
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), index_page,
				 label);

	sbox = gtk_hbox_new(FALSE, 0);
	gtk_widget_show(sbox);

	status_bar = gtk_label_new("");
	gtk_widget_show(status_bar);
	gtk_label_set_justify(GTK_LABEL(status_bar), GTK_JUSTIFY_LEFT);	
	gtk_box_pack_start(GTK_BOX(sbox), status_bar, FALSE, FALSE, 0);
	update_status("Select \"Camera->Get Index\" to preview images.");

	progress = gtk_progress_bar_new();
	gtk_widget_show(progress);
	gtk_box_pack_end(GTK_BOX(sbox), progress, FALSE, FALSE, 0);

	style = gtk_widget_get_style(mainWin);
	pixmap = gdk_pixmap_create_from_xpm_d(mainWin->window, &bitmap,
					&style->bg[GTK_STATE_NORMAL],
					(gchar **)splash_xpm);
	gpixmap = gtk_pixmap_new(pixmap, bitmap);
	gtk_widget_show(gpixmap);

	/* Main window layout ------------------------------------ */
	table =gtk_table_new(4,1,FALSE);
	gtk_container_add(GTK_CONTAINER(mainWin), table);
	gtk_widget_show(table);
	gtk_table_attach(GTK_TABLE(table),menu_bar, 0, 1, 0, 1,
			 GTK_FILL|GTK_EXPAND, GTK_FILL, 0 , 0);
	gtk_table_attach(GTK_TABLE(table), box, 0, 1, 1, 2,
			 GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0);
	gtk_table_attach_defaults(GTK_TABLE(table),notebook, 0, 1, 2, 3);
	gtk_table_attach(GTK_TABLE(table),sbox, 0, 1, 3, 4,
			 GTK_FILL|GTK_EXPAND, GTK_FILL, 0 , 0);

	index_table = gtk_hbox_new(FALSE, 0);
        gtk_widget_show(index_table);
        gtk_container_add(GTK_CONTAINER(index_window), index_table); 
	gtk_box_pack_start(GTK_BOX(index_table), gpixmap, TRUE, FALSE, 0);

	gtk_widget_show(mainWin);
	if (no_rc)
		port_dialog();
	gtk_main();
	return(0);
}

